#!/bin/sh

export LC_ALL=POSIX
unset CFLAGS CC CXX CPP LD_LIBRARY_PATH LD_PRELOAD
DIR=`pwd`

echo "=== 1.1-make-tools.sh"

. "$DIR"/sources/scripts/runuml.sh

echo "=== Phase 1: cross-compile a uClibc based toolchain."

# This script builds a /tools directory as described in Linux From Scratch
# V5.0 (www.linuxfromscratch.org).

# /tools provides a self-contained compiler that can build simple software
# packages on an otherwise empty system.  This provides us with a known
# environment with which to build the final system, one which can be removed
# by deleting a single directory.

# We're using uClibc instead of glibc, we've replaced the gnu tools (except
# for compiler) with busybox, and we package the result in a squashfs image.

# If we're not running as root, we wrap the tools build in User Mode Linux.
# The UML init script sets up the environment so the build script thinks it's
# running as root in a normal Linux system.


if [ `id -u` -eq 0 ]
then
  rm /tools 2>/dev/null
  ln -s `pwd`/tmpdir/tools /tools &&
  sources/scripts/1.1-tools-build.sh &&
  rm /tools/finished &&
  rm /tools &&
  cd tmpdir &&
  tar cvjf tools.tbz tools &&
  ./mksquashfs tools tools.img -info &&
  cd ..
else
  if [ ! -f tmpdir/workspace.img] || [ ! -f tmpdir/swap.img ]
  then
    sources/scripts/make-workspace.sh
  fi
  UMLSETUP=sources/scripts/1.0-tools-umlsetup.sh \
  runuml sources/scripts/1.1-tools-build.sh
fi

if [ ! -f tmpdir/tools.* ]; then exit 1; fi
