#!/tools/bin/sh

function makelink
{
  [ -L "$2" ] || ln -s "$1" "$2"
}

echo === Make directories

# This is a bit idiosyncratic: some redundant directories have been merged,
# with symlinks from the standard positions pointing to their new
# positions.

# Make directory hierarchy with /{bin,sbin,lib} symlinked to /usr/{bin,sbin,lib}

mkdir -p {dev/{shm,pts},home,mnt,proc,sys} &&
mkdir -p usr/{bin,etc,include,lib,sbin,src,share/{dict,doc,info,locale,man/man{1,2,3,4,5,6,7,8},nls,misc,terminfo,zoneinfo}} &&
mkdir -p var/{lock,log,mail,run,spool,tmp,opt,cache,lib/misc,local} &&
mkdir -p opt/{bin,doc,include,info,lib,man/man{1,2,3,4,5,6,7,8}} &&
chmod 1777 var/tmp &&
# Collapse together some redundant directories (nonstandard, but compatable.)
mkdir -p var/{boot,etc/opt} &&
makelink var/boot boot &&
makelink var/etc etc &&
makelink var/tmp tmp &&
makelink share/man usr/man &&
makelink share/doc usr/doc &&
makelink share/info usr/info &&
makelink .. usr/local &&
makelink usr/bin bin &&
makelink usr/sbin sbin &&
makelink usr/lib lib

if [ $? -ne 0 ]; then exit 1; fi

# Temporary symlinks (should all be removed/overwritten during build)
# It's ok if these fail.

ln -s /tools/bin/{bash,cat,pwd,stty} bin
ln -s /tools/lib/{libc.so.0,ld-uClibc.so.0} lib
ln -s bash /bin/sh

echo "=== Populate /etc (passwd,group,TZ,protocols,services) " &&
cat>/etc/passwd <<EOF &&
root:x:0:0:root:/root:/bin/bash
EOF
cat>/etc/group<<EOF &&
root:x:0:
bin:x:1:
sys:x:2:
kmem:x:3:
tty:x:4:
tape:x:5:
daemon:x:6:
floppy:x:7:
disk:x:8:
lp:x:9:
dialout:x:10:
audio:x:11:
EOF
ln -sf /proc/mounts /etc/mtab &&
echo CST6EDT > /etc/TZ

#TODO
#install -m 644 /sources/protocols /etc/protocols &&
#install -m 644 /sources/services /etc/services

if [ $? -ne 0 ]; then exit 1; fi

echo === Unpack linux kernel headers somewhere uclibc can find them.

cd /tmp &&
tar xvjf /sources/buildtools/linux-libc-headers.tar.bz2 &&
cd linux-libc-headers*/include &&
mv asm-$arch ../../asm &&
mv linux sound ../.. &&
cd ../.. &&
rm -rf linux-libc-headers*

if [ $? -ne 0 ]; then exit 1; fi

echo === Install uclibc

cd /tmp &&
tar xvjf /sources/system/uClibc.tar.bz2 &&
cd uClibc* &&
cat /sources/configs/uClibc-config > .config &&
# fix_includes gets deeply confused by pre-prepared kernel headers, so
# do its job manually.
echo > extra/scripts/fix_includes.sh &&
mv ../asm ../linux ../sound include &&
make &&
#ln -s /tools/lib/libgcc_s.so.1 /usr/lib &&
make PREFIX=/usr/ install_runtime &&
make install_dev &&
# Also install ldd and friends
make CFLAGS=-static install_utils &&
cd .. &&
#rm /usr/lib/libgcc_s.so.1 &&
rm -rf uClibc*

if [ $? -ne 0 ]; then exit 1; fi

echo === oneit
gcc -Os -s /sources/system/oneit.c -o /sbin/oneit || exit 1

# Base operating system

echo === busybox

/sources/scripts/build-busybox.sh initramfs &&
/sources/scripts/build-busybox.sh firmware &&
mv firmware-busybox /bin &&
cd busybox* &
make PREFIX=/ install &&
cd .. &&
rm -rf busybox*

if [ $? -ne 0 ]; then exit 1; fi

# Stuff that busybox should be upgraded to replace...

echo === bash

cd /tmp &&
tar xvzf /sources/temp/bash.tar.gz &&
cd bash* &&
cat /sources/temp/bash*patch | patch -p1 &&
# Yes, gcc 4 broke bash by gratuitously treating as an error something
# that should only be a warning.  Don't ask me why bash has its' own malloc
# implementation anyway.  Welcome to dealing with fsf code...
sed '/free_return:/a do {;} while(0);' -i lib/malloc/malloc.c &&
./configure --prefix=/usr --bindir=/bin &&
make &&
make install &&
cd .. &&
rm -rf bash*

if [ $? -ne 0 ]; then exit 1; fi

echo === diffutils

cd /tmp &&
tar xvzf /sources/temp/diffutils.tar.gz &&
cd diffutils* &&
./configure --prefix=/usr &&
make &&
make install &&
cd .. &&
rm -rf diffutils*

if [ $? -ne 0 ]; then exit 1; fi
