#!/bin/sh

cd "$DIR"/tmpdir &&

# Mmake a partitioned hard drive image file that qemu can boot.

CYLINDERS=64
HEADS=16
SECTORS=63
START=1

dd if=/dev/zero of=hda.img bs=$[$HEADS*$SECTORS*512] count=$CYLINDERS &&
#sfdisk -uS -C $CYLINDERS -H $HEADS -S $SECTORS  hda.img << EOF &&
#$START,$[$HEADS*SECTORS*$CYLINDERS-1],83
#EOF

# This is going to give an error because fdisk is stupid and can't be
# told not to call the reread partition table IOCTL.

fdisk -C $CYLINDERS -H $HEADS -S $SECTORS hda.img << EOF
c
n
p
1


w
EOF

# Attach loop1 to whole disk, loop2 to first partition
losetup /dev/loop1 hda.img &&
losetup -o $[$START*512] /dev/loop2 hda.img &&
# Block size 1024 so sectors size is sectors/2
mke2fs -F /dev/loop2 -b 1024 $[$[$CYLINDERS*$HEADS*$SECTORS-$START]/2] &&
tune2fs -c 0 -i 0 /dev/loop2 &&
mount -t ext2 /dev/loop2 mnt &&
cp ../firmware-boot mnt &&

# Run lilo against image

LENGTH=`dd if=mnt/firmware-boot skip=12 bs=1 count=4 2> /dev/null | od -l | head -n 1 | awk '{print $2}'` &&
cat > mnt/lilo.conf << EOF &&
boot=/dev/loop1
compact
delay=10
timeout=999
disk=/dev/loop1
  bios=0x80
  cylinders=128
  heads=16
  sectors=63
  partition=/dev/loop2
    start=1

image=mnt/firmware-boot
        label=linux
        root=/dev/loop2
        length=$LENGTH
        append="FIRMWARE=hda/hda1:firmware-boot"
        read-write
EOF
lilo -C mnt/lilo.conf -m mnt/lilo.map -s mnt/bootage &&
umount mnt &&
losetup -d /dev/loop1 &&
# Busybox umount will have detached this already.  Non-busybox umount won't.  I
# could supply -D to umount, but non-busybox umount wouldn't know what that is.
losetup -d /dev/loop2 2> /dev/null

