#!/bin/sh

# Find the UML executable in the parent filesystem
mount -n -t proc /proc /proc
mount -n -t sysfs /sys /sys
mount -n -t hostfs -o / /mnt /mnt 2>/dev/null
if [ $? -eq 0 ]
then
  firmware_path=/mnt`readlink /mnt/proc/self/exe`
else
  # For a bootable linux kernel, use /sys to find /dev/hda1/path/to/vmlinuz
  if [ -z "$FIRMWARE" ];
  then
    echo You must specify FIRMWARE=hda1:/path/to/firmware on the kernel command line.
    sleep 9999999
  fi
  firmware_dev=`echo $FIRMWARE | sed 's/:.*//'`
  firmware_path=/mnt/`echo $FIRMWARE | sed 's/.*://'`
  minor=`cat /sys/block/$firmware_dev/dev | sed 's/.*://'`
  major=`cat /sys/block/$firmware_dev/dev | sed 's/:.*//'`
  mknod -m 700 /block b $major $minor
  mount -o ro /block /mnt
fi
# Find offset of squashfs image.
len=`dd if=$firmware_path skip=12 bs=1 count=4 2> /dev/null | od -l | head -n 1 | awk '{print $2}'`
# loopback mount it
losetup -o $len /dev/loop0 $firmware_path
mount -n -t squashfs -o ro /dev/loop0 /dev
unset firmware_dev firmware_path len
cd dev
mount --move /sys ./sys
mount --move /proc ./proc
# Need switch_root here to free memory from rootfs
mount --move /dev /
exec chroot . /bin/sh
